@@ -7,13 +7,13 @@ no warnings;
 use subs qw();
 use vars qw($VERSION);
 
-$VERSION = '1.043';
+$VERSION = '1.045';
 
 =encoding utf8
 
 =head1 NAME
 
-Module::Extract::Use - Pull out the modules a module explicitly uses
+Module::Extract::Use - Discover the modules a module explicitly uses
 
 =head1 SYNOPSIS
 
@@ -36,9 +36,53 @@ Module::Extract::Use - Pull out the modules a module explicitly uses
 Extract the names of the modules used in a file using a static
 analysis. Since this module does not run code, it cannot find dynamic
 uses of modules, such as C<eval "require $class">. It only reports modules
-that the file loads directly. Modules loaded with C<parent> or C<base>,
-for instance, will will be in the import list for those pragmas but
-won't have separate entries in the data this module returns.
+that the file loads directly or are in the import lists for L<parent>
+or L<base>.
+
+The module can handle the conventional inclusion of modules with either
+C<use> or C<require> as the statement:
+
+	use Foo;
+	require Foo;
+
+	use Foo 1.23;
+	use Foo qw(this that);
+
+It now finds C<require> as an expression, which is useful to lazily
+load a module once (and may be faster):
+
+	sub do_something {
+		state $rc = require Foo;
+		...
+		}
+
+Additionally, it finds module names used with C<parent> and C<base>,
+either of which establish an inheritance relationship:
+
+	use parent qw(Foo);
+	use base qw(Foo);
+
+In the case of namespaces found in C<base> or C<parent>, the value of
+the C<direct> method is false. In all other cases, it is true. You
+can then skip those namespaces:
+
+	my $details = $extor->get_modules_with_details( $file );
+	foreach my $detail ( @$details ) {
+		next unless $detail->direct;
+
+		...
+		}
+
+This module does not discover runtime machinations to load something,
+such as string evals:
+
+	eval "use Foo";
+
+	my $bar = 'Bar';
+	eval "use $bar";
+
+If you want that, you might consider L<Module::ExtractUse> (a confusingly
+similar name).
 
 =cut
 
@@ -73,12 +117,12 @@ sub init {
 
 =item get_modules( FILE )
 
-Returns a list of namespaces explicity use-d in FILE. Returns undef if the
-file does not exist or if it can't parse the file.
+Returns a list of namespaces explicity use-d in FILE. Returns the
+empty list if the file does not exist or if it can't parse the file.
 
-Each used namespace is only in the list even if it is used multiple times
-in the file. The order of the list does not correspond to anything so don't
-use the order to infer anything.
+Each used namespace is only in the list even if it is used multiple
+times in the file. The order of the list does not correspond to
+anything so don't use the order to infer anything.
 
 =cut
 
@@ -88,11 +132,10 @@ sub get_modules {
 	$self->_clear_error;
 
 	my $details = $self->get_modules_with_details( $file );
-	return unless defined $details;
 
-	my @modules =
-		map { $_->{module} }
-		@$details;
+	my @modules = map { $_->module } @$details;
+
+	@modules;
 	}
 
 =item get_modules_with_details( FILE )
@@ -104,6 +147,7 @@ explicitly use-d in FILE. Each reference has keys for:
 	version   - defined if a module version was specified
 	imports   - an array reference to the import list
 	pragma    - true if the module thinks this namespace is a pragma
+	direct    - false if the module name came from parent or base
 
 Each used namespace is only in the list even if it is used multiple
 times in the file. The order of the list does not correspond to
@@ -117,7 +161,7 @@ sub get_modules_with_details {
 	$self->_clear_error;
 
 	my $modules = $self->_get_ppi_for_file( $file );
-	return unless defined $modules;
+	return [] unless defined $modules;
 
 	$modules;
 	}
@@ -138,20 +182,45 @@ sub _get_ppi_for_file {
 		return;
 		}
 
+	# this handles the
+	#   use Foo;
+	#   use Bar;
+	my $regular_modules = $self->_regular_load( $Document );
+
+	# this handles
+	#   use parent qw(...)
+	my $isa_modules      = $self->_isa_load( $regular_modules );
+
+	# this handles
+	#   my $rc = require Foo;
+	my $expression_loads = $self->_expression_load( $Document );
+
+	my @modules = map { @$_ }
+		$regular_modules,
+		$isa_modules,
+		$expression_loads
+		;
+
+	return \@modules;
+	}
+
+sub _regular_load {
+	my( $self, $Document ) = @_;
+
 	my $modules = $Document->find(
 		sub {
-			$_[1]->isa( 'PPI::Statement::Include' )  &&
-				( $_[1]->type eq 'use' || $_[1]->type eq 'require' )
+			$_[1]->isa( 'PPI::Statement::Include' )
 			}
 		);
 
-	return unless $modules;
+	return [] unless $modules;
 
 	my %Seen;
 	my @modules =
 		grep { ! $Seen{ $_->{module} }++ && $_->{module} }
 		map  {
 			my $hash = bless {
+				direct  => 1,
 				content => $_->content,
 				pragma  => $_->pragma,
 				module  => $_->module,
@@ -160,12 +229,64 @@ sub _get_ppi_for_file {
 				}, 'Module::Extract::Use::Item';
 			} @$modules;
 
-	return \@modules;
+	\@modules;
+	}
+
+sub _isa_load {
+	my( $self, $modules ) = @_;
+	my @isa_modules =
+		map {
+			my $m = $_;
+			map {
+				bless {
+					content => $m->content,
+					pragma  => '',
+					direct  => 0,
+					module  => $_,
+					imports => [],
+					version => undef,
+					}, 'Module::Extract::Use::Item';
+				} @{ $m->imports };
+			}
+		grep { $_->module eq 'parent' or $_->module eq 'base' }
+		@$modules;
+
+	\@isa_modules;
+	}
+
+sub _expression_load {
+	my( $self, $Document ) = @_;
+
+	my $in_statements = $Document->find(
+		sub {
+			$_[1]->isa( 'PPI::Token::Word' ) &&
+			$_[1]->content eq 'require'
+			}
+		);
+
+	return [] unless $in_statements;
+
+	my @modules =
+		map {
+			bless {
+				content => $_->parent->content,
+				pragma  => undef,
+				direct  => 1,
+				module  => $_->snext_sibling->content,
+				imports => [],
+				version => undef,
+				}, 'Module::Extract::Use::Item';
+			}
+		@$in_statements;
+
+	\@modules;
 	}
 
 BEGIN {
 package Module::Extract::Use::Item;
 
+sub direct  { $_[0]->{direct}  }
+sub content { $_[0]->{content} }
 sub pragma  { $_[0]->{pragma}  }
 sub module  { $_[0]->{module}  }
 sub imports { $_[0]->{imports} }
@@ -209,21 +330,15 @@ sub error        { $_[0]->{error} }
 
 =head1 TO DO
 
-=over 4
-
-=item * Make it recursive, so it scans the source for any module that it finds.
-
-=back
-
 =head1 SEE ALSO
 
-L<Module::ScanDeps>
+L<Module::ScanDeps>, L<Module::Extract>
 
 =head1 SOURCE AVAILABILITY
 
 The source code is in Github:
 
-	git://github.com/briandfoy/module-extract-use.git
+	https://github.com/briandfoy/module-extract-use
 
 =head1 AUTHOR
 
@@ -231,7 +346,7 @@ brian d foy, C<< <bdfoy@cpan.org> >>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2008-2017, brian d foy C<< <bdfoy@cpan.org> >>. All rights reserved.
+Copyright © 2008-2020, brian d foy C<< <bdfoy@cpan.org> >>. All rights reserved.
 
 This project is under the Artistic License 2.0.
 
